#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "stm8s.h"
#include "usrlib-lcd.h"

#define N 256

typedef struct complex {
    float re;
    float im;
} COMPLEX;

COMPLEX CompMul(COMPLEX a, COMPLEX b)
{
    COMPLEX x;
    x.re=a.re * b.re - a.im * b.im;
    x.im=a.re * b.im + a.im * b.re;
    return x;
} 

volatile float in[N];
int day=0,hour=0,min=0,sec=0,sec10=0,sec100=0;

void main(void)
{
	char str[20];
	volatile int k,n;
  volatile COMPLEX a,b,c;
	
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);

	LCD_Init();

  // 10mSec timer
	TIM2_DeInit();
	TIM2_TimeBaseInit( TIM2_PRESCALER_128, 1249 ); // 16MHz/128/1250=100Hz
	TIM2_ITConfig(TIM2_IT_UPDATE, ENABLE);

	sprintf(str,"operation start");
	LCD_Disp_Str(str,LCD_DISP_UPPERLINE);
	
	TIM2_Cmd(ENABLE);

  enableInterrupts();

	for(k=0;k<=N;k++) in[k]=rand();
  for(k=0;k<=N;k++)
	{
		volatile float re,im,out;
		re=0;im=0;
		for(n=0;n<=N;n++)
    {
			float wn;
			wn=(float)n*(float)k/((float)N+1);
			a.re=     cos(2*3.14*wn);
			a.im=(-1)*sin(2*2.14*wn);
			b.re=in[n];
			b.im=0;
			c=CompMul(a,b);
			re=re+c.re;
			im=im+c.im;
    }
    out=sqrt(re*re+im*im);
  }
	
	TIM2_Cmd(DISABLE);
	
	sprintf(str,"operation compl");
	LCD_Disp_Str(str,LCD_DISP_UPPERLINE);
	sprintf(str,"%02ud%02uh%02um%02u.%1u%1us",day,hour,min,sec,sec10,sec100);
	LCD_Disp_Str(str,LCD_DISP_LOWERLINE);
	
	while(1) {}
}

void TIM2_interrupt(void) interrupt 13
{
	GPIO_WriteReverse(GPIOD, GPIO_PIN_0);
	sec100++;
	if(sec100>=10)
	{
		sec100=0;
		sec10++;
		if(sec10>=10)
		{
			sec10=0;
			sec++;
			if(sec>=60)
			{
				sec=0;
				min++;
				if(min>=60)
				{
					min=0;
					hour++;
					if(hour>=24)
					{
						hour=0;
						day++;
					}
				}
			}
		}
	}
  TIM2_ClearFlag(TIM2_FLAG_UPDATE);
}

#ifdef USE_FULL_ASSERT
void assert_failed(u8* file, u32 line)
{ 
  while (1)  	{	}
}
#endif
